// ConnectGwSheet.cpp : implementation file
//

#include "stdafx.h"
#include "rtcuprog.h"
#include "ConnectGwSheet.h"
#include <process.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern bool gConnected;

/////////////////////////////////////////////////////////////////////////////
// CConnectGwSheet

IMPLEMENT_DYNAMIC(CConnectGwSheet, CPropertySheet)

CConnectGwSheet::CConnectGwSheet():CPropertySheet("Connect via RTCU Gateway", NULL, 0)
{
	// Not attempting to connect
	m_attempting = false;

	// Setup pages
	AddPage(&m_Page1);
	AddPage(&m_Page2);
	AddPage(&m_Page3);
}

CConnectGwSheet::~CConnectGwSheet()
{
}


BEGIN_MESSAGE_MAP(CConnectGwSheet, CPropertySheet)
	//{{AFX_MSG_MAP(CConnectGwSheet)
	ON_WM_DESTROY()
	ON_COMMAND(IDCANCEL, OnClose)
	//}}AFX_MSG_MAP
	ON_REGISTERED_MESSAGE(UWM_CONNECT_SHEET, OnConnectSheet)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Helpers for saving/restoring window state

static TCHAR BASED_CODE szSection[] = _T("GWConnectSettings");
static TCHAR BASED_CODE szWindowPos[] = _T("WindowPos");
static TCHAR szFormat[] = _T("%u,%u,%d,%d,%d,%d,%d,%d,%d,%d");

static BOOL PASCAL NEAR ReadWindowPlacement(LPWINDOWPLACEMENT pwp)
{
	CString strBuffer = AfxGetApp()->GetProfileString(szSection, szWindowPos);
	if (strBuffer.IsEmpty())
		return FALSE;

	WINDOWPLACEMENT wp;
	int nRead = sscanf_s(strBuffer, szFormat,
		&wp.flags, &wp.showCmd,
		&wp.ptMinPosition.x, &wp.ptMinPosition.y,
		&wp.ptMaxPosition.x, &wp.ptMaxPosition.y,
		&wp.rcNormalPosition.left, &wp.rcNormalPosition.top,
		&wp.rcNormalPosition.right, &wp.rcNormalPosition.bottom);

	if (nRead != 10)
		return FALSE;

	wp.length = sizeof wp;
	*pwp = wp;
	return TRUE;
}

static void PASCAL NEAR WriteWindowPlacement(LPWINDOWPLACEMENT pwp)
	// write a window placement to settings section of app's ini file
{
	TCHAR szBuffer[sizeof("-32767")*8 + sizeof("65535")*2];

	wsprintf(szBuffer, szFormat,
		pwp->flags, pwp->showCmd,
		pwp->ptMinPosition.x, pwp->ptMinPosition.y,
		pwp->ptMaxPosition.x, pwp->ptMaxPosition.y,
		pwp->rcNormalPosition.left, pwp->rcNormalPosition.top,
		pwp->rcNormalPosition.right, pwp->rcNormalPosition.bottom);
	AfxGetApp()->WriteProfileString(szSection, szWindowPos, szBuffer);
}

/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Initiation / Destruction
/////////////////////////////////////////////////////////////////////////////

BOOL CConnectGwSheet::OnInitDialog() 
{
  // Var
  CString         text;
  CRect           rc;
  int             top;
  WINDOWPLACEMENT wp;

  // Default
  BOOL bResult = CPropertySheet::OnInitDialog();

  // Adjust pages
  for (int i = 0; i < GetPageCount(); ++i) {
    if (GetPage(i)->GetSafeHwnd()) {
      // Change size
      GetPage(i)->GetWindowRect(&rc);
      GetPage(i)->SetWindowPos(NULL, 0, 0, 212, rc.Height(), SWP_NOMOVE|SWP_NOOWNERZORDER|SWP_NOZORDER|SWP_NOACTIVATE);
    }
  }

  // Adjust TAB ctrl
  GetTabControl()->GetWindowRect(&rc);
  GetTabControl()->SetWindowPos(NULL, 0, 0, 222, rc.Height(), SWP_NOMOVE|SWP_NOOWNERZORDER|SWP_NOZORDER|SWP_NOACTIVATE);
  top = rc.Height() + 12;

  // Adjust sheet
  GetWindowRect(&rc);
  rc.right = rc.left + 240;
  SetWindowPos(NULL, 0, 0, rc.Width(), rc.Height() + 30, SWP_NOMOVE|SWP_NOOWNERZORDER|SWP_NOZORDER|SWP_NOACTIVATE);

  // Redraw sheet
  GetTabControl()->SetCurFocus(2);
  GetTabControl()->SetCurFocus(1);
  GetTabControl()->SetCurFocus(0);

  // Restore position
  GetWindowRect(&rc);
  if (ReadWindowPlacement(&wp)) {
    // Check that the size of the dialog hasn't changed (allow max 4 pixels in both directions)
    if (((rc.bottom-rc.top) & 0xFFFC) == ((wp.rcNormalPosition.bottom - wp.rcNormalPosition.top) & 0xFFFC) &&
        ((rc.right-rc.left) & 0xFFFC) == ((wp.rcNormalPosition.right - wp.rcNormalPosition.left) & 0xFFFC)) {
      wp.showCmd=false;
      SetWindowPlacement(&wp);
    }
  }

  // Set controls
  m_ctrclose.CreateEx(0, _T("BUTTON"), "Close", WS_CHILD|WS_VISIBLE|WS_TABSTOP, rc.Width()-93, top, 80, 23, m_hWnd, (HMENU)IDCANCEL, 0);
  m_ctrclose.SetFont(GetFont());

  // Completed
  return bResult;
}

void CConnectGwSheet::OnDestroy() 
{
	// before it is destroyed, save the position of the window
	WINDOWPLACEMENT wp;
	wp.length = sizeof wp;
	if (GetWindowPlacement(&wp)) {
		wp.flags = 0;
		if (IsZoomed())
			wp.flags |= WPF_RESTORETOMAXIMIZED;
		// and write it to the .INI file
		WriteWindowPlacement(&wp);
	}

	// Default
	CPropertySheet::OnDestroy();
}

void CConnectGwSheet::PostNcDestroy() 
{
	// Default
	CPropertySheet::PostNcDestroy();
	// Cleanup
	delete this;
}

void CConnectGwSheet::OnClose() 
{
	ShowWindow(false);
}

/////////////////////////////////////////////////////////////////////////////
// Connect / Disconnect
/////////////////////////////////////////////////////////////////////////////

LRESULT CConnectGwSheet::OnConnectSheet(WPARAM /*wParam*/, LPARAM lParam)
{
	// Connection attempt?
	if (lParam) {
		// Is there a Node ID?
		if (!m_Page1.m_comboselect.IsEmpty()) {
			// Var
			CString str;
			bool gw_change = false;

			// Update Gateway settings
			if (m_Page2.m_ctrgwportspin.GetSafeHwnd()) m_Page2.SendMessage(UWM_CONNECT_PAGE, 0, 0);
			if (m_Page3.m_gwspinmaxsend.GetSafeHwnd()) m_Page3.SendMessage(UWM_CONNECT_PAGE, 0, 0);

			// Set status, attempting
			m_Page1.SendMessage(UWM_CONNECT_PAGE, 1, 0);

			// Disconnect cable
			BeginWaitCursor();
			rmonDisconnect(hCon);

			// Check gateway settings
			if (gStateGW.Port != m_Page2.m_gwport) gw_change = true;
			if (gStateGW.IP != m_Page2.m_gwip) gw_change = true;
			if (gStateGW.Key != m_Page2.m_gwkey) gw_change = true;
			if (gStateGW.maxsend != m_Page3.m_gwmaxsend) gw_change = true;
			if (gStateGW.resptimeout != m_Page3.m_gwresptimeout) gw_change = true;
			for (int i = 0; i < 16; i++) {
				if (gStateGW.CryptKey[i] != m_Page3.m_crypt_key[i]) {
					gw_change = true;
					break;
				}
			}

			// Change in gateway parameters?
			if (gw_change) {
				DEBUGMSG("Reconfigure Gateway connection setup\n", str);

				// Close MONMS library
				rmonClose();

				// Use 0 as nodeid, we will be assigned a number from the Gateway
				// Set keepalive to 30 seconds
				rmonSetGWParameters((unsigned short) m_Page2.m_gwport, 0, (LPCTSTR)m_Page2.m_gwip, (LPCTSTR)m_Page2.m_gwkey);
				rmonSetGWParametersAdv(m_Page3.m_crypt_key, 3,(unsigned char) m_Page3.m_gwmaxsend, (unsigned short) m_Page3.m_gwresptimeout, 30);

				// Set new Gateway parameters
				gStateGW.Port        = (unsigned short) m_Page2.m_gwport;
				gStateGW.IP          = m_Page2.m_gwip;
				gStateGW.Key         = m_Page2.m_gwkey;
				gStateGW.maxsend     = (unsigned char) m_Page3.m_gwmaxsend;
				gStateGW.resptimeout = (unsigned short) m_Page3.m_gwresptimeout;
				memcpy(gStateGW.CryptKey, m_Page3.m_crypt_key, 16);

				// Open MONMS library
				rmonOpen();

				// Create session
				hCon = rmonOpenConnection();
				DEBUGMSG("Connection: %s\n", hCon == HRMONCON_ILLEGAL ? "Failed" : "Open");

				// Get comports
				CString p1, p2;
				p1 = AfxGetApp()->GetProfileString("Options", "CABLE COMPORT", "COM0");
				p2 = AfxGetApp()->GetProfileString("Options", "MODEM COMPORT", "COM0");

				// Comport config
				DEBUGMSG("Comports: Cable=[%s] Modem=[%s]\n", p1, p2);
				rmonSetComport(hCon, p1, p2);

				// Get modem init string
				str = AfxGetApp()->GetProfileString("Options", "MODEMINIT", "ATS80.6=1;ATB15 ! Zyxel ISDN Modem");
				if (str.Find('!') != -1)  // if there is a comment, only grab to the point before the comment
					str=str.Left(str.Find('!'));
				str.TrimLeft();
				str.TrimRight();

				// Modem config
				DEBUGMSG("ModemInit=[%s]\n", str);
				rmonSetModemInit(hCon, str);
				rmonSetRemoteBaudrate(hCon, atoi(AfxGetApp()->GetProfileString("Options", "MODEM BAUDRATE", "57600")));
			}

			// Set status, connecting
			m_Page1.SendMessage(UWM_CONNECT_PAGE, 2, 0);

			// Start connection thread
			m_attempting = true;
			_beginthread(thConnect, 0, this);
		}
	}
	// Disconnect
	else {
		// Stop connection attempt
		m_attempting = false;

		// Drop the line
		BeginWaitCursor();
		rmonDisconnect(hCon);
		EndWaitCursor();

		// Update connection page
		m_Page1.SendMessage(UWM_CONNECT_PAGE, 0, 0);

		// Reconnect cable
		rmonConnect(hCon, "");
	}

	// Completed
	return 0;
}

void CConnectGwSheet::thConnect(void *arg)
{
	// Var
	int thrc;

	// Connect to unit
	thrc = rmonConnect(hCon, "@"+((CConnectGwSheet*)arg)->m_Page1.m_comboselect);
	DEBUGMSG("rmonConnect()=%i\n", thrc);

	// Connection attempt completed
	((CConnectGwSheet*)arg)->m_attempting = false;

	// Failed to connect?
    if (thrc) {
		// Set status
		if (thrc == rmonComError) {
			((CConnectGwSheet*)arg)->m_Page1.PostMessage(UWM_CONNECT_PAGE, 3, (long)(LPCTSTR)((CConnectGwSheet*)arg)->m_Page2.m_gwip);
		} else {
			((CConnectGwSheet*)arg)->m_Page1.PostMessage(UWM_CONNECT_PAGE, 4, 0);
		}

		// Cleanup
		rmonDisconnect(hCon);

		// Reconnect cable
		rmonConnect(hCon, "");
    } else {
		// Wait for data transfer
		while (!gConnected && rmonConnected(hCon) == RMONCON_GW) Sleep(100);
		thrc = rmonConnected(hCon) == RMONCON_NONE ? 0 : 5;
		// Set status
		((CConnectGwSheet*)arg)->m_Page1.PostMessage(UWM_CONNECT_PAGE, thrc, 0);
	}
}
